
#include <list>
#include <sstream>
using namespace std;

#include <ocarina/gtk.h>
#include <ocarina/songlist.h>
#include <libsaria/controls.h>
#include <libsaria/print.h>
#include "songlist.h"

void songlist_selected_inode(GtkTreeView *treeview, GtkTreePath *path, sid_t &inode)
{
	GtkTreeIter iter;
	GtkTreeModel *model = gtk_tree_view_get_model(treeview);

	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_model_get(model, &iter, 0, &inode, -1);
}

void songlist_row_activated(GtkTreeView *treeview, GtkTreePath *path,
				   GtkTreeViewColumn *column, gpointer data)
{
	GtkTreeIter iter;
	sid_t inode;
	GtkTreeModel *model = gtk_tree_view_get_model(treeview);
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_model_get(model, &iter, 0, &inode, -1);
	println("Double click! %lu", inode);
	libsaria::play_id(inode);
}

static void selected_foreach_func(GtkTreeModel *model, GtkTreePath *path,
				  GtkTreeIter *iter,   gpointer data)
{
	list<sid_t> *res = (list<sid_t> *) data;
	sid_t inode;
	gtk_tree_model_get(model, iter, 0, &inode, -1);
	res->push_back(inode);
}

void SongList::list_selected_ids(list<sid_t> &res)
{
	gtk_tree_selection_selected_foreach(selection, selected_foreach_func, &res);
}

GtkWidget *SongList::get_window()
{
	return window;
}

GtkWidget *SongList::get_label()
{
	return label;
}

void SongList::set_label_text()
{
	string res;
	stringstream stream;

	stream << name << " (";
	stream << list_funcs->size() << ")";
	res = stream.str();

	gtk_label_set_text(GTK_LABEL(label), res.c_str());
}

struct scroll_data {
	sid_t inode;
	GtkWidget *treeview;
	GdkRectangle rect;
};

static int scroll_to_check_row(GtkTreeModel *model, GtkTreePath *path,
				GtkTreeIter  *iter,  gpointer data)
{
	struct scroll_data *scroll = (struct scroll_data *)data;
	sid_t current;
	GtkTreeViewColumn *col;
	GtkTreePath *target;
	GdkRectangle area;
	int n;

	gtk_tree_model_get(model, iter, 0, &current, -1);
	if (current != scroll->inode)
		return FALSE;

	col = gtk_tree_view_get_column(GTK_TREE_VIEW(scroll->treeview), 0);
	gtk_tree_view_get_cell_area(GTK_TREE_VIEW(scroll->treeview), path, col, &area);
	target = gtk_tree_path_copy(path);
	if (area.height > 0) {
		n = scroll->rect.height / area.height;
		for (int i = 0; i < (n / 2); i++)
			gtk_tree_path_prev(target);
	}

	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(scroll->treeview),
				     target, NULL, TRUE, 0, 0);
	gtk_tree_view_set_cursor_on_cell(GTK_TREE_VIEW(scroll->treeview),
					 path, NULL, NULL, FALSE);
	gtk_tree_path_free(target);
	return TRUE;
}

void SongList::scroll_to(libsaria::Track *track)
{
	struct scroll_data data;
	data.inode = track->get_songid();
	data.treeview = treeview;

	gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(treeview), &data.rect);
	/* Widget not realized yet */
	if (data.rect.height == 0)
		return;

	gtk_tree_model_foreach(filter, scroll_to_check_row, &data);
}
